using FL.Database;
using System;
using UnityEngine;
using XGame.Framework;
using XGame.Framework.Asset;
using XGame.Framework.FGUI;
using XGame.Framework.Interfaces;
using XGame.Framework.Quality;
using XGame.Framework.Time;
using XGame.Framework.UI;

namespace FL.Launch
{
    public class FLGameLogic : IGameLogic
    {
        #region ӿʵ

        private IQualityModule _qualityModule;
        private TimeModule _timeModule;
        private IUIModule _uiModule;

        void IGameLogic.Init()
        {
            Log.Debug("DemoGameLogic.Init");
            InitModules();
        }

        void IGameLogic.Start()
        {
            Log.Debug("DemoGameLogic.Start");
            var repoAsync = SimpleTableRepo.LoadAsync();
            repoAsync.On(_ =>
            {
                Log.Debug($"SimpleTableRepo .");
                var tables = SimpleTableRepo.GetAll();
                var table = SimpleTableRepo.Get(1);
            });
            var mapAsync = AssetManager.LoadAsync<GameObject>("testmainmap");
            mapAsync.On(_ =>
            {
                var result = mapAsync.GetResult();
                result.transform.SetParent(AppManager.Root, false);
            });
            var panelAsync = _uiModule.OpenAsync(FGUI.UIKeys.BagMain, "TestMainGameMain");
        }

        void IGameLogic.Update(int millisecond)
        {
            //Log.Debug($"DemoGameLogic.Update millisecond:{millisecond}");
            (_uiModule as IUpdate).Update(millisecond);
        }
        void IGameLogic.LateUpdate(int millisecond)
        {
            (_uiModule as ILateUpdate).LateUpdate(millisecond);
            //Log.Debug($"DemoGameLogic.LateUpdate millisecond:{millisecond}");
        }
        void IGameLogic.Dispose()
        {
            Log.Debug("DemoGameLogic.Dispose");
            EventSingle.Dispose();
            _timeModule.Dispose();
            (_uiModule as IDisposable)?.Dispose();
        }
        #endregion

        private void InitModules()
        {
            _qualityModule = new QualityModule();
            _timeModule = new TimeModule();
            _timeModule.Initialize();
            var builder = new UIModuleFGUIBuilder();
            builder.Build(_qualityModule, _timeModule);
            _uiModule = builder.Product;
        }
    }
}

