using XGame.Framework.Asyncs;

namespace XGame.Framework.Loadable
{
    /// <summary>
    /// 异步加载器
    /// </summary>
    public class AsyncLoadable : ILoadAsync
    {
        public LoadState State { get; protected set; }
        private AsyncGroup _loadGroup;
        private AsyncGroup _unloadGroup;

        public IAsync LoadStatus => _loadGroup;

        public IAsync UnloadStatus => _unloadGroup;

        public IAsync LoadAsync()
        {
            Assert.IsTrue(State == LoadState.None || State == LoadState.Unloaded, $"Connot LoadAsync On State {State} Loadable: {this} ");

            State = LoadState.Loading;

            _loadGroup = new AsyncGroup();

            try
            {
                OnLoadAsync(_loadGroup);
            }
            catch (System.Exception e)
            {
                Log.Exception($"AsyncLoadable.OnLoadAsync Error: \n", e);
            }

            if (_loadGroup != null)
            {
                _loadGroup.On(OnLoadGroupCompleted);
                _loadGroup.End();
            }

            return _loadGroup;
        }

        public IAsync UnloadAsync()
        {
            Assert.IsFalse(State != LoadState.Loading && State != LoadState.Loaded, $"Connot UnloadAsync On State {State}  Loadable: {this} ");

            if (State == LoadState.Loading)
            {
                _loadGroup?.CancelAsync();
            }
            State = LoadState.Unloading;

            _unloadGroup = new AsyncGroup();

            try
            {
                OnUnloadAsync(_unloadGroup);
            }
            catch (System.Exception e)
            {
                Log.Exception($"AsyncLoadable.OnUnloadAsync Error: \n", e);
            }

            if (_unloadGroup != null)
            {
                _unloadGroup.On(OnUnloadGroupCompleted);
                _unloadGroup.End();
            }

            return _unloadGroup;
        }

        private void OnLoadGroupCompleted(IAsync async = null)
        {
            State = LoadState.Loaded;
            try
            {
                OnLoaded();
            }
            catch (System.Exception e)
            {
                Log.Exception($"AsyncLoadable.OnLoaded Error: \n", e);
            }

        }

        private void OnUnloadGroupCompleted(IAsync async = null)
        {
            State = LoadState.Unloaded;
            try
            {
                OnUnloaded();
            }
            catch (System.Exception e)
            {
                Log.Exception($"AsyncLoadable.OnUnloaded Error: \n", e);
            }

        }

        protected virtual void OnLoaded() { }
        protected virtual void OnUnloaded() { }
        protected virtual void OnLoadAsync(IAsyncGroup group) { }
        protected virtual void OnUnloadAsync(IAsyncGroup group) { }
    }
}
