using System;
using XGame.Framework.Asyncs;
using XGame.Framework.Data;
using XGame.Framework.Interfaces;
using XGame.Framework.Loadable;

namespace XGame.Framework.Nodes
{
    /// <summary>
    /// Nodeڵ
    /// </summary>
    public abstract class Node : AsyncLoadable, INode, IUpdate, ILateUpdate, IDisposable
    {
        #region ӿʵ
        public bool Active { get; protected set; }
        public NodeContext Context { get; private set; }
        private DataGroup _dataGroup;
        private BusinessGroup _businessGroup;
        public Node(NodeContext context)
        {
            Context = context;
            Init();
        }
        private void Init()
        {
            _dataGroup = new DataGroup();
            _businessGroup = new BusinessGroup(Context);
            RegisterData(_dataGroup);
            RegisterBusiness(_businessGroup);
            OnInited();
        }
        void INode.Enable(object intent)
        {
            if (Active) return;
            if (State != LoadState.Loaded) return;

            Active = true;
            _businessGroup.Enable(intent);
        }

        void INode.Disable()
        {
            if (!Active) return;
            if (State != LoadState.Loaded) return;
            Active = false;
            _businessGroup.Disable();
        }
        void IUpdate.Update(int millisecond)
        {
            if (!Active) return;
            _businessGroup.Update(millisecond);
        }
        void ILateUpdate.LateUpdate(int millisecond)
        {
            if (!Active) return;
            _businessGroup.LateUpdate(millisecond);
        }
        void IDisposable.Dispose()
        {
            _businessGroup.Dispose();
            //(Context as IDisposable).Dispose();
            OnDispose();
        }
        #endregion

        protected override void OnLoadAsync(IAsyncGroup group)
        {
            _dataGroup.OnLoadAsync(group);
            _businessGroup.OnLoadAsync(group);
        }
        protected override void OnUnloadAsync(IAsyncGroup group)
        {
            _dataGroup.OnUnloadAsync(group);
            _businessGroup.OnUnloadAsync(group);
        }
        protected abstract void RegisterData(IDataGroup dataGroup);
        protected abstract void RegisterBusiness(IBusinessGroup businessGroup);

        protected abstract void OnInited();
        protected abstract void OnDispose();
    }

}
