﻿using System;
using XGame.Framework.Quality;
using XGame.Framework.Time;

namespace XGame.Framework.UI
{
    /// <summary>
    /// 每个UIView一个UIContext实例
    /// </summary>
    public class UIContext : IDisposable
    {
        public IQualityModule Quality { get; private set; }
        public ITimeModule Time { get; private set; }
        public IUIModule UI { get; private set; }
        public IUIAssetModule Asset { get; private set; }

        public UIContext(IQualityModule qualityModule, ITimeModule timeModule, IUIModule uIModule)
        {
            Quality = qualityModule;
            Time = timeModule;
            UI = uIModule;
            Asset = new UIAssetModule();
        }

        public UIContext Clone()
        {
            return new UIContext(Quality, Time, UI);
        }

        void IDisposable.Dispose()
        {
            Quality = null;
            Time = null;
            UI = null;
            (Asset as IDisposable)?.Dispose();
            Asset = null;
        }
    }

    public interface IUIContextSetter
    {
        UIContext Context { set; }
    }
}
